@@ -69,7 +69,13 @@ else:
 integer_types = six.integer_types + (np.integer,)
 
 
+# pandas compat
 try:
     from pandas.api.types import DatetimeTZDtype  # noqa: F401
 except ImportError:
     from pandas.types.dtypes import DatetimeTZDtype  # noqa: F401
+
+try:
+    from pandas.core.tools.datetimes import to_time  # noqa: F401
+except ImportError:
+    from pandas.tseries.tools import to_time  # noqa: F401
