@@ -25,8 +25,6 @@ import pprint
 import warnings
 import sys
 
-from six import StringIO
-
 PY2 = sys.version_info[0] == 2
 
 if not PY2:
@@ -155,28 +153,32 @@ class DictWrapper(object):
         object.__setattr__(self, "prefix", prefix)
 
     def __repr__(self):
-        buf = StringIO()
-        pprint.pprint(self.d, stream=buf)
-        return buf.getvalue()
+        return pprint.pformat(self.d)
 
     def __setattr__(self, key, val):
-        prefix = object.__getattribute__(self, "prefix")
+        prefix = self.prefix
         if prefix:
             prefix += "."
         prefix += key
-        # you can't set new keys
-        # can you can't overwrite subtrees
+
+        # you can't set new keys and you can't overwrite subtrees
+
         if key in self.d and not isinstance(self.d[key], dict):
             _set_option(prefix, val)
         else:
             raise OptionError("You can only set the value of existing options")
 
     def __getattr__(self, key):
-        prefix = object.__getattribute__(self, "prefix")
+        prefix = self.prefix
         if prefix:
             prefix += "."
         prefix += key
-        v = object.__getattribute__(self, "d")[key]
+
+        try:
+            v = self.d[key]
+        except KeyError as e:
+            raise AttributeError(*e.args)
+
         if isinstance(v, dict):
             return DictWrapper(v, prefix)
         else:
@@ -327,8 +329,11 @@ class option_context(object):
     You need to invoke as ``option_context(pat, val, [(pat, val), ...])``.
     Examples
     --------
-    >>> with option_context('display.max_rows', 10, 'display.max_columns', 5):
-            ...
+    >>> with option_context('interactive', True):
+    ...     print(options.interactive)
+    True
+    >>> options.interactive
+    False
     """
 
     def __init__(self, *args):
