@@ -12,11 +12,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from ibis.expr.tests.mocks import MockConnection
+import collections
 
 import pytest
+
 import ibis
 
+from ibis.expr.tests.mocks import MockConnection
+
 
 @pytest.fixture
 def schema():
@@ -29,17 +32,18 @@ def schema():
         ('f', 'double'),
         ('g', 'string'),
         ('h', 'boolean'),
+        ('i', 'timestamp'),
     ]
 
 
 @pytest.fixture
 def schema_dict(schema):
-    return dict(schema)
+    return collections.OrderedDict(schema)
 
 
 @pytest.fixture
 def table(schema):
-    return ibis.table(schema, name='schema')
+    return ibis.table(schema, name='table')
 
 
 @pytest.fixture(params=list('abcdh'))
@@ -70,3 +74,18 @@ def col(request):
 @pytest.fixture
 def con():
     return MockConnection()
+
+
+@pytest.fixture
+def alltypes(con):
+    return con.table('alltypes')
+
+
+@pytest.fixture
+def functional_alltypes(con):
+    return con.table('functional_alltypes')
+
+
+@pytest.fixture
+def lineitem(con):
+    return con.table('tpch_lineitem')
