@@ -18,6 +18,13 @@ def test_nested_array():
     ) == dt.Array(dt.Array(dt.string))
 
 
+def test_array_with_string_value_type():
+    assert dt.Array('int32') == dt.Array(dt.int32)
+    assert dt.Array(dt.Array('array<map<string, double>>')) == (
+        dt.Array(dt.Array(dt.Array(dt.Map(dt.string, dt.double))))
+    )
+
+
 def test_map():
     assert dt.validate_type(
         'map<string, double>'
@@ -30,6 +37,12 @@ def test_nested_map():
     ) == dt.Map(dt.int64, dt.Array(dt.Map(dt.string, dt.int8)))
 
 
+def test_map_with_string_value_type():
+    assert dt.Map('int32', 'double') == dt.Map(dt.int32, dt.double)
+    assert dt.Map('int32', 'array<double>') == \
+        dt.Map(dt.int32, dt.Array(dt.double))
+
+
 def test_map_does_not_allow_non_primitive_keys():
     with pytest.raises(SyntaxError):
         dt.validate_type('map<array<string>, double>')
@@ -79,9 +92,40 @@ def test_struct():
     assert dt.validate_type(orders) == expected
 
 
-def test_decimal_failure():
+def test_struct_with_string_types():
+    result = dt.Struct.from_tuples(
+        [
+            ('a', 'map<double, string>'),
+            ('b', 'array<map<string, array<int32>>>'),
+            ('c', 'array<string>'),
+            ('d', 'int8'),
+        ]
+    )
+
+    assert result == dt.Struct.from_tuples(
+        [
+            ('a', dt.Map(dt.double, dt.string)),
+            ('b', dt.Array(dt.Map(dt.string, dt.Array(dt.int32)))),
+            ('c', dt.Array(dt.string)),
+            ('d', dt.int8),
+        ]
+    )
+
+
+@pytest.mark.parametrize(
+    'case',
+    [
+        'decimal(',
+        'decimal()',
+        'decimal(3)',
+        'decimal(,)',
+        'decimal(3,)',
+        'decimal(3,',
+    ]
+)
+def test_decimal_failure(case):
     with pytest.raises(SyntaxError):
-        dt.validate_type('decimal(')
+        dt.validate_type(case)
 
 
 @pytest.mark.parametrize(
@@ -219,3 +263,75 @@ def test_literal_mixed_type_fails():
     data = [1, 'a']
     with pytest.raises(TypeError):
         ibis.literal(data)
+
+
+def test_array_type_not_equals():
+    left = dt.Array(dt.string)
+    right = dt.Array(dt.int32)
+
+    assert not left.equals(right)
+    assert left != right
+    assert not (left == right)
+
+
+def test_array_type_equals():
+    left = dt.Array(dt.string)
+    right = dt.Array(dt.string)
+
+    assert left.equals(right)
+    assert left == right
+    assert not (left != right)
+
+
+def test_timestamp_with_timezone_parser_single_quote():
+    t = dt.validate_type("timestamp('US/Eastern')")
+    assert isinstance(t, dt.Timestamp)
+    assert t.timezone == 'US/Eastern'
+
+
+def test_timestamp_with_timezone_parser_double_quote():
+    t = dt.validate_type("timestamp('US/Eastern')")
+    assert isinstance(t, dt.Timestamp)
+    assert t.timezone == 'US/Eastern'
+
+
+def test_timestamp_with_timezone_parser_invalid_timezone():
+    ts = dt.validate_type("timestamp('US/Ea')")
+    assert str(ts) == "timestamp('US/Ea')"
+
+
+@pytest.mark.parametrize(
+    'case',
+    [
+        "timestamp(US/Ea)",
+        "timestamp('US/Eastern\")",
+        'timestamp("US/Eastern\')',
+    ]
+)
+def test_timestamp_parsing_failure_mode(case):
+    with pytest.raises(SyntaxError):
+        dt.validate_type(case)
+
+
+def test_timestamp_with_invalid_timezone():
+    ts = dt.Timestamp('Foo/Bar&234')
+    assert str(ts) == "timestamp('Foo/Bar&234')"
+
+
+def test_timestamp_with_timezone_repr():
+    ts = dt.Timestamp('UTC')
+    assert repr(ts) == "Timestamp(timezone='UTC', nullable=True)"
+
+
+def test_timestamp_with_timezone_str():
+    ts = dt.Timestamp('UTC')
+    assert str(ts) == "timestamp('UTC')"
+
+
+def test_time():
+    ts = dt.time
+    assert str(ts) == "time"
+
+
+def test_time_valid():
+    assert dt.validate_type('time').equals(dt.time)
