@@ -16,7 +16,6 @@ import pytest
 
 from ibis.expr.types import ColumnExpr, TableExpr, RelationError
 from ibis.common import ExpressionError
-from ibis.expr.datatypes import array_type
 import ibis.expr.api as api
 import ibis.expr.types as ir
 import ibis.expr.operations as ops
@@ -66,7 +65,6 @@ def test_getitem_column_select(table, schema_dict):
 
         # Make sure it's the right type
         assert isinstance(col, ColumnExpr)
-        assert isinstance(col, array_type(v))
 
         # Ensure we have a field selection with back-reference to the table
         parent = col.parent()
@@ -277,6 +275,7 @@ def test_mutate_alter_existing_columns(table):
 
     expected = table['a', 'b', 'c', 'd', 'e',
                           new_f.name('f'), 'g', 'h',
+                          'i',
                           foo.name('foo')]
 
     assert_equal(expr, expected)
@@ -480,22 +479,26 @@ def test_mean_expr_basics(table, numeric_col):
 
 
 def test_aggregate_no_keys(table):
-    agg_exprs = [table['a'].sum().name('sum(a)'),
-                 table['c'].mean().name('mean(c)')]
+    metrics = [
+        table['a'].sum().name('sum(a)'),
+        table['c'].mean().name('mean(c)'),
+    ]
 
     # A TableExpr, which in SQL at least will yield a table with a single
     # row
-    result = table.aggregate(agg_exprs)
+    result = table.aggregate(metrics)
     assert isinstance(result, TableExpr)
 
 
 def test_aggregate_keys_basic(table):
-    agg_exprs = [table['a'].sum().name('sum(a)'),
-                 table['c'].mean().name('mean(c)')]
+    metrics = [
+        table['a'].sum().name('sum(a)'),
+        table['c'].mean().name('mean(c)'),
+    ]
 
     # A TableExpr, which in SQL at least will yield a table with a single
     # row
-    result = table.aggregate(agg_exprs, by=['g'])
+    result = table.aggregate(metrics, by=['g'])
     assert isinstance(result, TableExpr)
 
     # it works!
@@ -726,6 +729,24 @@ def test_join_no_predicate_list(con):
     assert_equal(joined, expected)
 
 
+def test_asof_join():
+    left = ibis.table([('time', 'int32'), ('value', 'double')])
+    right = ibis.table([('time', 'int32'), ('value2', 'double')])
+    joined = api.asof_join(left, right, 'time')
+    pred = joined.op().predicates[0].op()
+    assert pred.left.op().name == pred.right.op().name == 'time'
+
+
+def test_asof_join_with_by():
+    left = ibis.table(
+        [('time', 'int32'), ('key', 'int32'), ('value', 'double')])
+    right = ibis.table(
+        [('time', 'int32'), ('key', 'int32'), ('value2', 'double')])
+    joined = api.asof_join(left, right, 'time', by='key')
+    by = joined.op().by_predicates[0].op()
+    assert by.left.op().name == by.right.op().name == 'key'
+
+
 def test_equijoin_schema_merge():
     table1 = ibis.table([('key1',  'string'), ('value1', 'double')])
     table2 = ibis.table([('key2',  'string'), ('stuff', 'int32')])
@@ -873,9 +894,11 @@ def test_semi_join_schema(table):
 
 
 def test_cross_join(table):
-    agg_exprs = [table['a'].sum().name('sum_a'),
-                 table['b'].mean().name('mean_b')]
-    scalar_aggs = table.aggregate(agg_exprs)
+    metrics = [
+        table['a'].sum().name('sum_a'),
+        table['b'].mean().name('mean_b'),
+    ]
+    scalar_aggs = table.aggregate(metrics)
 
     joined = table.cross_join(scalar_aggs).materialize()
     agg_schema = api.Schema(['sum_a', 'mean_b'], ['int64', 'double'])
@@ -959,6 +982,18 @@ def test_join_invalid_refs(con):
         t1.inner_join(t2, [predicate])
 
 
+def test_join_invalid_expr_type(con):
+    left = con.table('star1')
+    invalid_right = left.foo_id
+    join_key = ['bar_id']
+
+    with pytest.raises(TypeError) as e:
+        left.inner_join(invalid_right, join_key)
+
+    message = str(e)
+    assert type(invalid_right).__name__ in message
+
+
 def test_join_non_boolean_expr(con):
     t1 = con.table('star1')
     t2 = con.table('star2')
