@@ -13,18 +13,24 @@
 # limitations under the License.
 
 import operator
+from collections import OrderedDict
+from operator import methodcaller
+from datetime import date, datetime, time
 
 import pytest
 
 from ibis.common import IbisTypeError
-import ibis.expr.api as api
-import ibis.expr.datatypes as dt
-import ibis.expr.types as ir
-import ibis.expr.operations as ops
+from ibis.expr import (
+    rules, api, datatypes as dt, types as ir, operations as ops
+)
 import ibis
+from ibis.compat import PY2
 
+from ibis import literal
 from ibis.tests.util import assert_equal
 
+import toolz
+
 
 def test_null():
     expr = ibis.literal(None)
@@ -94,6 +100,64 @@ def test_literal_with_different_type(value, expected_type):
     assert expr.type().equals(dt.validate_type(expected_type))
 
 
+@pytest.mark.parametrize(
+    ['value', 'expected_type', 'expected_class'],
+    [
+        (list('abc'), 'array<string>', ir.ArrayScalar),
+        ([1, 2, 3], 'array<int8>', ir.ArrayScalar),
+        ({'a': 1, 'b': 2, 'c': 3}, 'map<string, int8>', ir.MapScalar),
+        ({1: 2, 3: 4, 5: 6}, 'map<int8, int8>', ir.MapScalar),
+        (
+            {'a': [1.0, 2.0], 'b': [], 'c': [3.0]},
+            'map<string, array<double>>',
+            ir.MapScalar
+        ),
+        (
+            OrderedDict([
+                ('a', 1),
+                ('b', list('abc')),
+                ('c', OrderedDict([('foo', [1.0, 2.0])]))
+            ]),
+            'struct<a: int8, b: array<string>, c: struct<foo: array<double>>>',
+            ir.StructScalar
+        )
+    ]
+)
+def test_literal_complex_types(value, expected_type, expected_class):
+    expr = ibis.literal(value)
+    expr_type = expr.type()
+    assert expr_type.equals(dt.validate_type(expected_type))
+    assert isinstance(expr, expected_class)
+    assert isinstance(expr.op(), ir.Literal)
+    assert expr.op().value is value
+
+
+def test_struct_operations():
+    value = OrderedDict([
+        ('a', 1),
+        ('b', list('abc')),
+        ('c', OrderedDict([('foo', [1.0, 2.0])]))
+    ])
+    expr = ibis.literal(value)
+    assert isinstance(expr, ir.StructValue)
+    assert isinstance(expr.b, ir.ArrayValue)
+    assert isinstance(expr.a.op(), ops.StructField)
+
+
+def test_simple_map_operations():
+    value = {'a': [1.0, 2.0], 'b': [], 'c': [3.0]}
+    value2 = {'a': [1.0, 2.0], 'c': [3.0], 'd': [4.0, 5.0]}
+    expr = ibis.literal(value)
+    expr2 = ibis.literal(value2)
+    assert isinstance(expr, ir.MapValue)
+    assert isinstance(expr['b'].op(), ops.MapValueForKey)
+    assert isinstance(expr.length().op(), ops.MapLength)
+    assert isinstance(expr.keys().op(), ops.MapKeys)
+    assert isinstance(expr.values().op(), ops.MapValues)
+    assert isinstance((expr + expr2).op(), ops.MapConcat)
+    assert isinstance((expr2 + expr).op(), ops.MapConcat)
+
+
 @pytest.mark.parametrize(
     ['value', 'expected_type'],
     [
@@ -121,6 +185,13 @@ def test_literal_list():
     repr(expr)
 
 
+def test_literal_array():
+    what = []
+    expr = api.literal(what)
+    assert isinstance(expr, ir.ArrayValue)
+    assert expr.type().equals(dt.Array(dt.null))
+
+
 def test_mixed_arity(table):
     what = ["bar", table.g, "foo"]
     expr = api.as_value_expr(what)
@@ -195,36 +266,31 @@ def test_scalar_isin_list_with_array(table):
     assert isinstance(not_expr, ir.BooleanColumn)
 
 
-@pytest.fixture
-def dtable(con):
-    return con.table('functional_alltypes')
-
-
-def test_distinct_basic(dtable):
-    expr = dtable.distinct()
+def test_distinct_basic(functional_alltypes):
+    expr = functional_alltypes.distinct()
     assert isinstance(expr.op(), ops.Distinct)
     assert isinstance(expr, ir.TableExpr)
-    assert expr.op().table is dtable
+    assert expr.op().table is functional_alltypes
 
-    expr = dtable.string_col.distinct()
+    expr = functional_alltypes.string_col.distinct()
     assert isinstance(expr.op(), ops.DistinctColumn)
 
     assert isinstance(expr, ir.StringColumn)
 
 
 @pytest.mark.xfail(reason='NYT')
-def test_distinct_array_interactions(dtable):
+def test_distinct_array_interactions(functional_alltypes):
     # array cardinalities / shapes are likely to be different.
-    a = dtable.int_col.distinct()
-    b = dtable.bigint_col
+    a = functional_alltypes.int_col.distinct()
+    b = functional_alltypes.bigint_col
 
     with pytest.raises(ir.RelationError):
         a + b
 
 
-def test_distinct_count(dtable):
-    result = dtable.string_col.distinct().count()
-    expected = dtable.string_col.nunique().name('count')
+def test_distinct_count(functional_alltypes):
+    result = functional_alltypes.string_col.distinct().count()
+    expected = functional_alltypes.string_col.nunique().name('count')
     assert_equal(result, expected)
     assert isinstance(result.op(), ops.CountDistinct)
 
@@ -243,13 +309,15 @@ def test_distinct_unnamed_array_expr():
     repr(expr)
 
 
-def test_distinct_count_numeric_types(dtable):
-    metric = dtable.bigint_col.distinct().count().name('unique_bigints')
-    dtable.group_by('string_col').aggregate(metric)
+def test_distinct_count_numeric_types(functional_alltypes):
+    metric = functional_alltypes.bigint_col.distinct().count().name(
+        'unique_bigints'
+    )
+    functional_alltypes.group_by('string_col').aggregate(metric)
 
 
-def test_nunique(dtable):
-    expr = dtable.string_col.nunique()
+def test_nunique(functional_alltypes):
+    expr = functional_alltypes.string_col.nunique()
     assert isinstance(expr.op(), ops.CountDistinct)
 
 
@@ -857,3 +925,237 @@ def test_generic_value_api_no_arithmetic(value, operation):
 )
 def test_fillna_null(value, expected):
     assert ibis.NA.fillna(value).type().equals(expected)
+
+
+@pytest.mark.parametrize(
+    ('left', 'right'),
+    [
+        (literal('2017-04-01'), date(2017, 4, 2)),
+        (date(2017, 4, 2), literal('2017-04-01')),
+        (literal('2017-04-01 01:02:33'), datetime(2017, 4, 1, 1, 3, 34)),
+        (datetime(2017, 4, 1, 1, 3, 34), literal('2017-04-01 01:02:33')),
+    ]
+)
+@pytest.mark.parametrize(
+    'op',
+    [
+        operator.eq,
+        operator.ne,
+        operator.lt,
+        operator.le,
+        operator.gt,
+        operator.ge,
+        lambda left, right: ibis.timestamp(
+            '2017-04-01 00:02:34'
+        ).between(left, right),
+        lambda left, right: ibis.timestamp(
+            '2017-04-01'
+        ).cast(dt.date).between(left, right)
+    ]
+)
+def test_string_temporal_compare(op, left, right):
+    result = op(left, right)
+    assert result.type().equals(dt.boolean)
+
+
+@pytest.mark.parametrize(
+    ('value', 'type', 'expected_type_class'),
+    [
+        (2.21, 'decimal', dt.Decimal),
+        (3.14, 'double', dt.Double),
+        (4.2, 'int64', dt.Double),
+        (4, 'int64', dt.Int64),
+    ]
+)
+def test_decimal_modulo_output_type(value, type, expected_type_class):
+    t = ibis.table([('a', type)])
+    expr = t.a % value
+    assert isinstance(expr.type(), expected_type_class)
+
+
+@pytest.mark.parametrize(
+    ('left', 'right'),
+    [
+        (literal('10:00'), time(10, 0)),
+        (time(10, 0), literal('10:00')),
+    ]
+)
+@pytest.mark.parametrize(
+    'op',
+    [
+        operator.eq,
+        operator.ne,
+        operator.lt,
+        operator.le,
+        operator.gt,
+        operator.ge,
+    ]
+)
+@pytest.mark.skipif(PY2, reason="time comparsions not available on PY2")
+def test_time_compare(op, left, right):
+    result = op(left, right)
+    assert result.type().equals(dt.boolean)
+
+
+@pytest.mark.parametrize(
+    ('left', 'right'),
+    [
+        (literal('10:00'), date(2017, 4, 2)),
+        (literal('10:00'), datetime(2017, 4, 2, 1, 1)),
+        (literal('10:00'), literal('2017-04-01')),
+    ]
+)
+@pytest.mark.parametrize(
+    'op',
+    [
+        operator.eq,
+        operator.lt,
+        operator.le,
+        operator.gt,
+        operator.ge,
+        ]
+)
+def test_time_timestamp_invalid_compare(op, left, right):
+    result = op(left, right)
+    assert result.type().equals(dt.boolean)
+
+
+@pytest.mark.skipif(not PY2, reason="invalid compare of time on PY2")
+def test_time_invalid_compare_on_py2():
+
+    # we cannot actually compare datetime.time objects and literals
+    # in a deferred way in python 2, they short circuit in the CPython
+    result = operator.eq(time(10, 0), literal('10:00'))
+    assert not result
+
+
+def test_scalar_parameter_repr():
+    value = ibis.param(dt.timestamp, name='value')
+    assert repr(value) == 'value = ScalarParameter[timestamp]'
+
+    value_op = value.op()
+    assert repr(value_op) == "ScalarParameter(name='value', type=timestamp)"
+
+
+@pytest.mark.parametrize(
+    ('left', 'right', 'expected'),
+    [
+        (
+            # same value type, same name
+            ibis.param(dt.timestamp, name='value1'),
+            ibis.param(dt.timestamp, name='value1'),
+            True,
+        ),
+        (
+            # different value type, same name
+            ibis.param(dt.date, name='value1'),
+            ibis.param(dt.timestamp, name='value1'),
+            False,
+        ),
+        (
+            # same value type, different name
+            ibis.param(dt.timestamp, name='value1'),
+            ibis.param(dt.timestamp, name='value2'),
+            False,
+        ),
+        (
+            # different value type, different name
+            ibis.param(dt.date, name='value1'),
+            ibis.param(dt.timestamp, name='value2'),
+            False,
+        ),
+        (
+            # different Python class, left side is param
+            ibis.param(dt.timestamp, 'value'),
+            dt.date,
+            False
+        ),
+        (
+            # different Python class, right side is param
+            dt.date,
+            ibis.param(dt.timestamp, 'value'),
+            False
+        ),
+    ]
+)
+def test_scalar_parameter_compare(left, right, expected):
+    assert left.equals(right) == expected
+
+
+@pytest.mark.parametrize(
+    ('case', 'creator'),
+    [
+        (datetime.now(), toolz.compose(methodcaller('time'), ibis.timestamp)),
+        ('now', toolz.compose(methodcaller('time'), ibis.timestamp)),
+        (datetime.now().time(), ibis.time),
+        ('10:37', ibis.time),
+    ]
+)
+@pytest.mark.parametrize(
+    ('left', 'right'),
+    [
+        (1, 'a'),
+        ('a', 1),
+        (1.0, 2.0),
+        (['a'], [1]),
+    ]
+)
+@pytest.mark.xfail(PY2, reason='Not supported on Python 2')
+def test_between_time_failure_time(case, creator, left, right):
+    value = creator(case)
+    with pytest.raises(TypeError):
+        value.between(left, right)
+
+
+def test_custom_type_binary_operations():
+    class Foo(ir.ValueExpr):
+
+        def __add__(self, other):
+            op = self.op()
+            return type(op)(op.value + other).to_expr()
+
+        __radd__ = __add__
+
+    class FooNode(ops.ValueOp):
+
+        input_type = [ibis.expr.rules.integer(name='value')]
+
+        def output_type(self):
+            return Foo
+
+    left = ibis.literal(2)
+    right = FooNode(3).to_expr()
+    result = left + right
+    assert isinstance(result, Foo)
+    assert isinstance(result.op(), FooNode)
+
+    left = FooNode(3).to_expr()
+    right = ibis.literal(2)
+    result = left + right
+    assert isinstance(result, Foo)
+    assert isinstance(result.op(), FooNode)
+
+
+def test_empty_array_as_argument():
+    class Foo(ir.Expr):
+        pass
+
+    class FooNode(ops.ValueOp):
+
+        input_type = [rules.array(dt.int64, name='value')]
+
+        def output_type(self):
+            return Foo
+
+    node = FooNode([])
+    value = node.value
+    expected = literal([]).cast(dt.Array(dt.int64))
+    assert not value.type().equals(dt.Array(dt.null))
+    assert value.type().equals(dt.Array(dt.int64))
+    assert value.equals(expected)
+
+
+def test_struct_field_dir():
+    t = ibis.table([('struct_col', 'struct<my_field: string>')])
+    assert 'struct_col' in dir(t)
+    assert 'my_field' in dir(t.struct_col)
