@@ -12,8 +12,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import collections
 import datetime
+import itertools
+import os
+import sys
+import warnings
+import webbrowser
+
 import six
+import toolz
 
 from ibis.common import IbisError, RelationError
 import ibis.common as com
@@ -23,18 +31,6 @@ import ibis.util as util
 import ibis.expr.datatypes as dt
 
 
-class Parameter(object):
-
-    """
-    Placeholder, to be implemented
-    """
-
-    pass
-
-
-# ---------------------------------------------------------------------
-
-
 class Expr(object):
 
     """
@@ -73,17 +69,45 @@ class Expr(object):
         from ibis.expr.format import ExprFormatter
         return ExprFormatter(self, memo=memo).get_result()
 
-    def pipe(self, f, *args, **kwargs):
+    def _repr_png_(self):
+        try:
+            import ibis.expr.visualize as viz
+        except ImportError:
+            return None
+        else:
+            try:
+                return viz.to_graph(self).pipe(format='png')
+            except Exception:
+                # Something may go wrong, and we can't error in the notebook
+                # so fallback to the default text representation.
+                return None
+
+    def visualize(self, format='png'):
+        """Visualize an expression in the browser as a PNG image.
+
+        Parameters
+        ----------
+        format : str, optional
+            Defaults to ``'png'``. Some additional formats are
+            ``'jpeg'`` and ``'svg'``. These are specified by the ``graphviz``
+            Python library.
+
+        Notes
+        -----
+        This method opens a web browser tab showing the image of the expression
+        graph created by the code in :module:`ibis.expr.visualize`.
+
+        Raises
+        ------
+        ImportError
+            If ``graphviz`` is not installed.
         """
-        Generic composition function to enable expression pipelining
+        import ibis.expr.visualize as viz
+        path = viz.draw(viz.to_graph(self), format=format)
+        webbrowser.open('file://{}'.format(os.path.abspath(path)))
 
-        >>> (expr
-        >>>  .pipe(f, *args, **kwargs)
-        >>>  .pipe(g, *args2, **kwargs2))
-
-        is equivalent to
-
-        >>> g(f(expr, *args, **kwargs), *args2, **kwargs2)
+    def pipe(self, f, *args, **kwargs):
+        """Generic composition function to enable expression pipelining.
 
         Parameters
         ----------
@@ -96,12 +120,33 @@ class Expr(object):
 
         Examples
         --------
-        >>> def foo(data, a=None, b=None):
-        ...     pass
-        >>> def bar(a, b, data=None):
-        ...     pass
-        >>> expr.pipe(foo, a=5, b=10)
-        >>> expr.pipe((bar, 'data'), 1, 2)
+        >>> import ibis
+        >>> t = ibis.table([('a', 'int64'), ('b', 'string')], name='t')
+        >>> f = lambda a: (a + 1).name('a')
+        >>> g = lambda a: (a * 2).name('a')
+        >>> result1 = t.a.pipe(f).pipe(g)
+        >>> result1  # doctest: +NORMALIZE_WHITESPACE
+        ref_0
+        UnboundTable[table]
+          name: t
+          schema:
+            a : int64
+            b : string
+        a = Multiply[int64*]
+          left:
+            a = Add[int64*]
+              left:
+                a = Column[int64*] 'a' from table
+                  ref_0
+              right:
+                Literal[int8]
+                  1
+          right:
+            Literal[int8]
+              2
+        >>> result2 = g(f(t.a))  # equivalent to the above
+        >>> result1.equals(result2)
+        True
 
         Returns
         -------
@@ -129,7 +174,7 @@ class Expr(object):
     def _can_implicit_cast(self, arg):
         return False
 
-    def execute(self, limit='default', async=False):
+    def execute(self, limit='default', async=False, params=None):
         """
         If this expression is based on physical tables in a database backend,
         execute it against that backend.
@@ -146,9 +191,9 @@ class Expr(object):
           Result of compiling expression and executing in backend
         """
         from ibis.client import execute
-        return execute(self, limit=limit, async=async)
+        return execute(self, limit=limit, async=async, params=params)
 
-    def compile(self, limit=None):
+    def compile(self, limit=None, params=None):
         """
         Compile expression to whatever execution target, to verify
 
@@ -158,7 +203,7 @@ class Expr(object):
            query representation or list thereof
         """
         from ibis.client import compile
-        return compile(self, limit=limit)
+        return compile(self, limit=limit, params=params)
 
     def verify(self):
         """
@@ -166,9 +211,10 @@ class Expr(object):
         """
         try:
             self.compile()
-            return True
-        except:
+        except Exception:
             return False
+        else:
+            return True
 
     def equals(self, other, cache=None):
         if type(self) != type(other):
@@ -187,6 +233,12 @@ class Expr(object):
         pass
 
 
+if sys.version_info.major == 2:
+    # Python 2.7 doesn't return NotImplemented unless the other operand has
+    # an attribute called "timetuple". This is a bug that's fixed in Python 3
+    Expr.timetuple = None
+
+
 def _safe_repr(x, memo=None):
     return x._repr(memo=memo) if isinstance(x, (Expr, Node)) else repr(x)
 
@@ -205,7 +257,7 @@ class OperationMeta(type):
                         continue
 
                     if t.name not in dct:
-                        dct[t.name] = _arg_getter(i)
+                        dct[t.name] = _arg_getter(i, doc=t.doc)
 
         return super(OperationMeta, cls).__new__(cls, name, parents, dct)
 
@@ -309,7 +361,6 @@ class Node(six.with_metaclass(OperationMeta, object)):
         if self._expr_cached is None:
             self._expr_cached = self._make_expr()
         return self._expr_cached
-        # return self._make_expr()
 
     def _make_expr(self):
         klass = self.output_type()
@@ -343,16 +394,13 @@ def all_equal(left, right, cache=None):
 
     if hasattr(left, 'equals'):
         return left.equals(right, cache=cache)
-    else:
-        return left == right
-    return True
+    return left == right
 
 
-def _arg_getter(i):
-    @property
+def _arg_getter(i, doc=None):
     def arg_accessor(self):
         return self.args[i]
-    return arg_accessor
+    return property(arg_accessor, doc=doc)
 
 
 class ValueOp(Node):
@@ -378,9 +426,12 @@ class TableColumn(ValueOp):
     """
 
     def __init__(self, name, table_expr):
-        Node.__init__(self, [name, table_expr])
+        schema = table_expr.schema()
+        if isinstance(name, six.integer_types):
+            name = schema.name_at_position(name)
 
-        if name not in table_expr.schema():
+        super(TableColumn, self).__init__(name, table_expr)
+        if name not in schema:
             raise com.IbisTypeError(
                 "'{0}' is not a field in {1}".format(name, table_expr.columns)
             )
@@ -458,21 +509,62 @@ class ExprList(Expr):
         return ExpressionList(exprs).to_expr()
 
 
+def infer_literal_type(value):
+    import ibis.expr.rules as rules
+
+    if value is None or value is null:
+        return dt.null
+    elif isinstance(value, bool):
+        return dt.boolean
+    elif isinstance(value, compat.integer_types):
+        return rules.int_literal_class(value)
+    elif isinstance(value, float):
+        return dt.double
+    elif isinstance(value, six.string_types):
+        return dt.string
+    elif isinstance(value, datetime.datetime):
+        return dt.timestamp
+    elif isinstance(value, datetime.date):
+        return dt.date
+    elif isinstance(value, datetime.time):
+        return dt.time
+    elif isinstance(value, list):
+        if not value:
+            return dt.Array(dt.null)
+        return dt.Array(rules.highest_precedence_type(
+            list(map(literal, value))
+        ))
+    elif isinstance(value, collections.OrderedDict):
+        if not value:
+            raise TypeError('Empty struct type not supported')
+        return dt.Struct(
+            list(value.keys()),
+            [literal(element).type() for element in value.values()],
+        )
+    elif isinstance(value, dict):
+        if not value:
+            return dt.Map(dt.null, dt.null)
+        return dt.Map(
+            rules.highest_precedence_type(list(map(literal, value.keys()))),
+            rules.highest_precedence_type(list(map(literal, value.values()))),
+        )
+
+    raise com.InputTypeError(value)
+
+
 class Literal(ValueOp):
 
-    def __init__(self, value):
+    def __init__(self, value, type=None):
+        super(Literal, self).__init__(value, type)
         self.value = value
+        self._output_type = type.scalar_type()
 
     def __repr__(self):
-        return '{0}({1})'.format(
+        return '{}({})'.format(
             type(self).__name__,
             ', '.join(map(repr, self.args))
         )
 
-    @property
-    def args(self):
-        return [self.value]
-
     def equals(self, other, cache=None):
         return (
             isinstance(other, Literal) and
@@ -481,41 +573,86 @@ class Literal(ValueOp):
         )
 
     def output_type(self):
-        import ibis.expr.rules as rules
-
-        value = self.value
-
-        if isinstance(value, bool):
-            return BooleanScalar
-        elif isinstance(value, compat.integer_types):
-            return rules.int_literal_class(value).scalar_type()
-        elif isinstance(value, float):
-            return DoubleScalar
-        elif isinstance(value, six.string_types):
-            return StringScalar
-        elif isinstance(value, datetime.datetime):
-            return TimestampScalar
-        elif isinstance(value, datetime.date):
-            return DateScalar
-        elif isinstance(value, list):
-            value_type = rules.highest_precedence_type(
-                list(map(literal, value))
-            )
-            return lambda value, value_type=value_type: ArrayScalar(
-                value, dt.Array(value_type)
-            )
-
-        raise com.InputTypeError(value)
+        return self._output_type
 
     def root_tables(self):
         return []
 
 
+_parameter_counter = itertools.count()
+
+
+def _parameter_name():
+    return 'param[{:d}]'.format(next(_parameter_counter))
+
+
+class ScalarParameter(ValueOp):
+
+    def __init__(self, type, name=None):
+        super(ScalarParameter, self).__init__(type)
+        self.name = name
+        self.output_type = type.scalar_type
+
+    def __repr__(self):
+        return '{}(name={!r}, type={})'.format(
+            type(self).__name__, self.name, self.type
+        )
+
+    @property
+    def type(self):
+        return self.args[0]
+
+    def equals(self, other, cache=None):
+        return (
+            isinstance(other, ScalarParameter) and
+            self.name == other.name and
+            self.type.equals(other.type, cache=cache)
+        )
+
+    def root_tables(self):
+        return []
+
+    def resolve_name(self):
+        return self.name
+
+
+def param(type, name=None):
+    """Create a parameter of a particular type to be defined just before
+    execution.
+
+    Parameters
+    ----------
+    type : dt.DataType
+        The type of the unbound parameter, e.g., double, int64, date, etc.
+    name : str, optional
+        The name of the parameter
+
+    Returns
+    -------
+    ScalarExpr
+
+    Examples
+    --------
+    >>> import ibis
+    >>> import ibis.expr.datatypes as dt
+    >>> start = ibis.param(dt.date)
+    >>> end = ibis.param(dt.date)
+    >>> schema = [('timestamp_col', 'timestamp'), ('value', 'double')]
+    >>> t = ibis.table(schema)
+    >>> predicates = [t.timestamp_col >= start, t.timestamp_col <= end]
+    >>> expr = t.filter(predicates).value.sum()
+    """
+    if name is None:
+        name = _parameter_name()
+    expr = ScalarParameter(dt.validate_type(type), name=name).to_expr()
+    return expr.name(name)
+
+
 def distinct_roots(*args):
     all_roots = []
     for arg in args:
         all_roots.extend(arg._root_tables())
-    return util.unique_by_key(all_roots, id)
+    return list(toolz.unique(all_roots, key=id))
 
 
 # ---------------------------------------------------------------------
@@ -629,9 +766,10 @@ class TableExpr(Expr):
     def _is_valid(self, exprs):
         try:
             self._assert_valid(util.promote_list(exprs))
-            return True
-        except:
+        except com.RelationError:
             return False
+        else:
+            return True
 
     def _assert_valid(self, exprs):
         from ibis.expr.analysis import ExprValidator
@@ -641,7 +779,7 @@ class TableExpr(Expr):
         return name in self.schema()
 
     def __getitem__(self, what):
-        if isinstance(what, six.string_types):
+        if isinstance(what, six.string_types + six.integer_types):
             return self.get_column(what)
 
         if isinstance(what, slice):
@@ -674,7 +812,10 @@ class TableExpr(Expr):
             # Projection convenience
             return self.projection(what)
         else:
-            raise NotImplementedError
+            raise NotImplementedError(
+                'Selection rows or columns with {} objects is not '
+                'supported'.format(type(what).__name__)
+            )
 
     def __len__(self):
         raise com.ExpressionError('Use .count() instead')
@@ -691,7 +832,7 @@ class TableExpr(Expr):
     def __dir__(self):
         attrs = dir(type(self))
         if self._is_materialized():
-            attrs = list(sorted(set(attrs + self.schema().names)))
+            attrs = list(frozenset(attrs + self.schema().names))
         return attrs
 
     def _resolve(self, exprs):
@@ -707,6 +848,8 @@ class TableExpr(Expr):
     def _ensure_expr(self, expr):
         if isinstance(expr, six.string_types):
             return self[expr]
+        elif isinstance(expr, six.integer_types):
+            return self[self.schema().name_at_position(expr)]
         elif not isinstance(expr, Expr):
             return expr(self)
         else:
@@ -721,7 +864,17 @@ class TableExpr(Expr):
 
         Examples
         --------
-        a, b, c = table.get_columns(['a', 'b', 'c'])
+        >>> import ibis
+        >>> table = ibis.table(
+        ...    [
+        ...        ('a', 'int64'),
+        ...        ('b', 'string'),
+        ...        ('c', 'timestamp'),
+        ...        ('d', 'float'),
+        ...    ],
+        ...    name='t'
+        ... )
+        >>> a, b, c = table.get_columns(['a', 'b', 'c'])
 
         Returns
         -------
@@ -754,7 +907,7 @@ class TableExpr(Expr):
         """
         if not self._is_materialized():
             raise IbisError('Table operation is not yet materialized')
-        return self.op().get_schema()
+        return self.op().schema
 
     def _is_materialized(self):
         # The operation produces a known schema
@@ -786,7 +939,11 @@ class TableExpr(Expr):
 
         Examples
         --------
-        x.group_by([b1, b2]).aggregate(metrics)
+        >>> import ibis
+        >>> pairs = [('a', 'int32'), ('b', 'timestamp'), ('c', 'double')]
+        >>> t = ibis.table(pairs)
+        >>> b1, b2 = t.a, t.b
+        >>> result = t.group_by([b1, b2]).aggregate(sum_of_c=t.c.sum())
 
         Notes
         -----
@@ -900,20 +1057,37 @@ class StringValue(AnyValue):
         return dt.string
 
     def _can_compare(self, other):
-        return isinstance(other, StringValue)
+        return isinstance(other, (StringValue, TemporalValue))
 
 
-class DecimalValue(NumericValue):
-
-    _implicit_casts = set(['float', 'double'])
-
-    def __init__(self, meta):
-        self.meta = meta
-        self._precision = meta.precision
-        self._scale = meta.scale
+class BinaryValue(AnyValue):
 
     def type(self):
-        return dt.Decimal(self._precision, self._scale)
+        return dt.binary
+
+    def _can_compare(self, other):
+        return isinstance(other, BinaryValue)
+
+
+class ParameterizedValue(AnyValue):
+
+    def __init__(self, meta, name=None):
+        super(ParameterizedValue, self).__init__(meta, name=name)
+        self.meta = meta
+
+    def type(self):
+        return self.meta
+
+    @property
+    def _factory(self):
+        def factory(arg, name=None):
+            return type(self)(arg, self.meta, name=name)
+        return factory
+
+
+class DecimalValue(ParameterizedValue, NumericValue):
+
+    _implicit_casts = set(['float', 'double'])
 
     @classmethod
     def _make_constructor(cls, meta):
@@ -922,7 +1096,12 @@ class DecimalValue(NumericValue):
         return constructor
 
 
-class DateValue(AnyValue):
+class TemporalValue(AnyValue):
+    def _can_compare(self, other):
+        return isinstance(other, (TemporalValue, StringValue))
+
+
+class DateValue(TemporalValue):
 
     def type(self):
         return dt.date
@@ -938,23 +1117,62 @@ class DateValue(AnyValue):
                 return False
         return False
 
-    def _can_compare(self, other):
-        return isinstance(other, DateValue)
-
     def _implicit_cast(self, arg):
         # assume we've checked this is OK at this point...
         op = arg.op()
         return DateScalar(op)
 
 
-class TimestampValue(AnyValue):
+class TimeValue(TemporalValue):
 
     def type(self):
-        return dt.timestamp
+        return dt.time
 
     def _can_implicit_cast(self, arg):
         op = arg.op()
         if isinstance(op, Literal):
+            try:
+                from ibis.compat import to_time
+                to_time(op.value)
+                return True
+            except ValueError:
+                return False
+        return False
+
+    def _can_compare(self, other):
+        return isinstance(other, (TimeValue, StringValue))
+
+    def _implicit_cast(self, arg):
+        # assume we've checked this is OK at this point...
+        op = arg.op()
+        return TimeScalar(op)
+
+
+class TimestampValue(TemporalValue):
+
+    def __init__(self, meta=None):
+        self.meta = meta
+        self._timezone = getattr(meta, 'timezone', None)
+
+    def type(self):
+        return dt.Timestamp(timezone=self._timezone)
+
+    @classmethod
+    def _make_constructor(cls, meta):
+        def constructor(arg, name=None):
+            return cls(arg, meta, name=name)
+        return constructor
+
+    def _can_implicit_cast(self, arg):
+        op = arg.op()
+        if isinstance(op, Literal):
+            if isinstance(op.value, six.integer_types):
+                warnings.warn(
+                    'Integer values for timestamp literals are deprecated in '
+                    '0.11.0 and will be removed in 0.12.0. To pass integers '
+                    'as timestamp literals, use '
+                    'pd.Timestamp({:d}, unit=...)'.format(op.value)
+                )
             try:
                 import pandas as pd
                 pd.Timestamp(op.value)
@@ -963,27 +1181,56 @@ class TimestampValue(AnyValue):
                 return False
         return False
 
-    def _can_compare(self, other):
-        return isinstance(other, TimestampValue)
-
     def _implicit_cast(self, arg):
         # assume we've checked this is OK at this point...
         op = arg.op()
         return TimestampScalar(op)
 
 
-class ArrayValue(AnyValue):
-
-    def __init__(self, type, name=None):
-        super(ArrayValue, self).__init__(type.value_type)
-        self.value_type = type.value_type
-
-    def type(self):
-        return dt.Array(self.value_type)
+class ArrayValue(ParameterizedValue):
 
     def _can_compare(self, other):
         return isinstance(other, ArrayValue)
 
+    def _can_cast_implicit(self, typename):
+        if not isinstance(typename, dt.Array):
+            return False
+
+        self_type = self.type()
+        return (
+            super(ArrayValue, self)._can_cast_implicit(typename) or
+            self_type.equals(dt.Array(dt.null)) or
+            self_type.equals(dt.Array(dt.any))
+        )
+
+
+class MapValue(ParameterizedValue):
+
+    def _can_compare(self, other):
+        return isinstance(other, MapValue)
+
+    def _can_cast_implicit(self, typename):
+        if not isinstance(typename, dt.Map):
+            return False
+
+        self_type = self.type()
+        return (
+            super(MapValue, self)._can_cast_implicit(typename) or
+            self_type.equals(dt.Map(dt.null, dt.null)) or
+            self_type.equals(dt.Map(dt.any, dt.any))
+        )
+
+
+class StructValue(ParameterizedValue):
+
+    def _can_compare(self, other):
+        return isinstance(other, StructValue)
+
+    def __dir__(self):
+        parent_attributes = dir(type(self))
+        field_names = self.type().names
+        return list(frozenset(itertools.chain(parent_attributes, field_names)))
+
 
 class NumericColumn(ColumnExpr, NumericValue):
     pass
@@ -1064,6 +1311,14 @@ class StringColumn(ColumnExpr, StringValue):
     pass
 
 
+class BinaryScalar(ScalarExpr, BinaryValue):
+    pass
+
+
+class BinaryColumn(ColumnExpr, BinaryValue):
+    pass
+
+
 class DateScalar(ScalarExpr, DateValue):
     pass
 
@@ -1072,12 +1327,26 @@ class DateColumn(ColumnExpr, DateValue):
     pass
 
 
-class TimestampScalar(ScalarExpr, TimestampValue):
+class TimeScalar(ScalarExpr, TimeValue):
     pass
 
 
+class TimeColumn(ColumnExpr, TimeValue):
+    pass
+
+
+class TimestampScalar(ScalarExpr, TimestampValue):
+
+    def __init__(self, arg, meta=None, name=None):
+        ScalarExpr.__init__(self, arg, name=name)
+        TimestampValue.__init__(self, meta=meta)
+
+
 class TimestampColumn(ColumnExpr, TimestampValue):
-    pass
+
+    def __init__(self, arg, meta=None, name=None):
+        ColumnExpr.__init__(self, arg, name=name)
+        TimestampValue.__init__(self, meta=meta)
 
 
 class DecimalScalar(DecimalValue, ScalarExpr):
@@ -1086,12 +1355,6 @@ class DecimalScalar(DecimalValue, ScalarExpr):
         DecimalValue.__init__(self, meta)
         ScalarExpr.__init__(self, arg, name=name)
 
-    @property
-    def _factory(self):
-        def factory(arg, name=None):
-            return DecimalScalar(arg, self.meta, name=name)
-        return factory
-
 
 class DecimalColumn(DecimalValue, NumericColumn):
 
@@ -1099,14 +1362,8 @@ class DecimalColumn(DecimalValue, NumericColumn):
         DecimalValue.__init__(self, meta)
         NumericColumn.__init__(self, arg, name=name)
 
-    @property
-    def _factory(self):
-        def factory(arg, name=None):
-            return DecimalColumn(arg, self.meta, name=name)
-        return factory
 
-
-class CategoryValue(AnyValue):
+class CategoryValue(ParameterizedValue):
 
     """
     Represents some ordered data categorization; tracked as an int32 value
@@ -1115,12 +1372,6 @@ class CategoryValue(AnyValue):
 
     _implicit_casts = Int16Value._implicit_casts
 
-    def __init__(self, meta):
-        self.meta = meta
-
-    def type(self):
-        return self.meta
-
     def _can_compare(self, other):
         return isinstance(other, IntegerValue)
 
@@ -1131,12 +1382,6 @@ class CategoryScalar(CategoryValue, ScalarExpr):
         CategoryValue.__init__(self, meta)
         ScalarExpr.__init__(self, arg, name=name)
 
-    @property
-    def _factory(self):
-        def factory(arg, name=None):
-            return CategoryScalar(arg, self.meta, name=name)
-        return factory
-
 
 class CategoryColumn(CategoryValue, ColumnExpr):
 
@@ -1144,12 +1389,6 @@ class CategoryColumn(CategoryValue, ColumnExpr):
         CategoryValue.__init__(self, meta)
         ColumnExpr.__init__(self, arg, name=name)
 
-    @property
-    def _factory(self):
-        def factory(arg, name=None):
-            return CategoryColumn(arg, self.meta, name=name)
-        return factory
-
 
 class ArrayScalar(ArrayValue, ScalarExpr):
 
@@ -1157,12 +1396,6 @@ class ArrayScalar(ArrayValue, ScalarExpr):
         ArrayValue.__init__(self, meta)
         ScalarExpr.__init__(self, arg, name=name)
 
-    @property
-    def _factory(self):
-        def factory(arg, name=None):
-            return ArrayScalar(arg, self.type(), name=name)
-        return factory
-
 
 class ArrayColumn(ArrayValue, ColumnExpr):
 
@@ -1170,11 +1403,33 @@ class ArrayColumn(ArrayValue, ColumnExpr):
         ArrayValue.__init__(self, meta)
         ColumnExpr.__init__(self, arg, name=name)
 
-    @property
-    def _factory(self):
-        def factory(arg, name=None):
-            return ArrayColumn(arg, self.type(), name=name)
-        return factory
+
+class MapScalar(MapValue, ScalarExpr):
+
+    def __init__(self, arg, meta, name=None):
+        MapValue.__init__(self, meta)
+        ScalarExpr.__init__(self, arg, name=name)
+
+
+class MapColumn(MapValue, ColumnExpr):
+
+    def __init__(self, arg, meta, name=None):
+        MapValue.__init__(self, meta)
+        ColumnExpr.__init__(self, arg, name=name)
+
+
+class StructScalar(StructValue, ScalarExpr):
+
+    def __init__(self, arg, meta, name=None):
+        StructValue.__init__(self, meta)
+        ScalarExpr.__init__(self, arg, name=name)
+
+
+class StructColumn(StructValue, ColumnExpr):
+
+    def __init__(self, arg, meta, name=None):
+        StructValue.__init__(self, meta)
+        ColumnExpr.__init__(self, arg, name=name)
 
 
 class UnnamedMarker(object):
@@ -1197,22 +1452,56 @@ def as_value_expr(val):
     return val
 
 
-def literal(value):
-    """
-    Create a scalar expression from a Python value
+def literal(value, type=None):
+    """Create a scalar expression from a Python value.
 
     Parameters
     ----------
     value : some Python basic type
+        A Python value
+    type : ibis type or string, optional
+        An instance of :class:`ibis.expr.datatypes.DataType` or a string
+        indicating the ibis type of `value`. This parameter should only be used
+        in cases where ibis's type inference isn't sufficient for discovering
+        the type of `value`.
 
     Returns
     -------
-    lit_value : value expression, type depending on input value
+    literal_value : Literal
+        An expression representing a literal value
+
+    Examples
+    --------
+    >>> import ibis
+    >>> x = ibis.literal(42)
+    >>> x.type()
+    int8
+    >>> y = ibis.literal(42, type='double')
+    >>> y.type()
+    double
+    >>> ibis.literal('foobar', type='int64')  # doctest: +ELLIPSIS
+    Traceback (most recent call last):
+      ...
+    TypeError: Value 'foobar' cannot be safely coerced to int64
     """
-    if value is None or value is null:
-        return null()
+    if hasattr(value, 'op') and isinstance(value.op(), Literal):
+        return value
+
+    if type is None:
+        type = infer_literal_type(value)
     else:
-        return Literal(value).to_expr()
+        type = dt.validate_type(type)
+
+    if not type.valid_literal(value):
+        raise TypeError(
+            'Value {!r} cannot be safely coerced to {}'.format(value, type)
+        )
+
+    if value is None or value is _NULL or value is null:
+        result = null().cast(type)
+    else:
+        result = Literal(value, type=type).to_expr()
+    return result
 
 
 _NULL = None
