@@ -22,7 +22,6 @@ import traceback
 from posixpath import join as pjoin
 from collections import deque
 
-import hdfs
 import numpy as np
 import pandas as pd
 
@@ -117,10 +116,13 @@ class ImpalaConnection(object):
 
         try:
             cursor.execute(query, async=async)
-        except:
-            exc = traceback.format_exc()
+        except Exception:
             cursor.release()
-            self.error('Exception caused by {}: {}'.format(query, exc))
+            self.error(
+                'Exception caused by {}: {}'.format(
+                    query, traceback.format_exc()
+                )
+            )
             raise
 
         return cursor
@@ -462,6 +464,7 @@ class ImpalaClient(SQLClient):
     async_query = ImpalaAsyncQuery
 
     def __init__(self, con, hdfs_client=None, **params):
+        import hdfs
         self.con = con
 
         if isinstance(hdfs_client, hdfs.Client):
@@ -476,8 +479,8 @@ class ImpalaClient(SQLClient):
 
         self._ensure_temp_db_exists()
 
-    def _build_ast(self, expr):
-        return build_ast(expr)
+    def _build_ast(self, expr, params=None):
+        return build_ast(expr, params=params)
 
     def _get_hdfs(self):
         if self._hdfs is None:
@@ -826,7 +829,7 @@ class ImpalaClient(SQLClient):
 
         Examples
         --------
-        con.create_table('new_table_name', table_expr)
+        >>> con.create_table('new_table_name', table_expr)  # doctest: +SKIP
         """
         if like_parquet is not None:
             raise NotImplementedError
@@ -1070,10 +1073,11 @@ class ImpalaClient(SQLClient):
 
         Examples
         --------
-        con.insert('my_table', table_expr)
+        >>> table = 'my_table'
+        >>> con.insert(table, table_expr)  # doctest: +SKIP
 
         # Completely overwrite contents
-        con.insert('my_table', table_expr, overwrite=True)
+        >>> con.insert(table, table_expr, overwrite=True)  # doctest: +SKIP
         """
         table = self.table(table_name, database=database)
         return table.insert(obj=obj, overwrite=overwrite, partition=partition,
@@ -1107,7 +1111,9 @@ class ImpalaClient(SQLClient):
 
         Examples
         --------
-        con.drop_table('my_table', database='operations', force=True)
+        >>> table = 'my_table'
+        >>> db = 'operations'
+        >>> con.drop_table(table, database=db, force=True)  # doctest: +SKIP
         """
         statement = ddl.DropTable(table_name, database=database,
                                   must_exist=not force)
@@ -1134,7 +1140,7 @@ class ImpalaClient(SQLClient):
         except Exception as e:
             try:
                 self.drop_view(name, database=database)
-            except:
+            except Exception:
                 raise e
 
     def cache_table(self, table_name, database=None, pool='default'):
@@ -1150,7 +1156,10 @@ class ImpalaClient(SQLClient):
 
         Examples
         --------
-        con.cache_table('my_table', database='operations', pool='op_4GB_pool')
+        >>> table = 'my_table'
+        >>> db = 'operations'
+        >>> pool = 'op_4GB_pool'
+        >>> con.cache_table('my_table', database=db, pool=pool)  # noqa: E501 # doctest: +SKIP
         """
         statement = ddl.CacheTable(table_name, database=database, pool=pool)
         self._execute(statement)
@@ -1655,10 +1664,10 @@ class ImpalaTable(ir.TableExpr, DatabaseEntity):
 
         Examples
         --------
-        t.insert(table_expr)
+        >>> t.insert(table_expr)  # doctest: +SKIP
 
         # Completely overwrite contents
-        t.insert(table_expr, overwrite=True)
+        >>> t.insert(table_expr, overwrite=True)  # doctest: +SKIP
         """
         if isinstance(obj, pd.DataFrame):
             from ibis.impala.pandas_interop import write_temp_dataframe
