@@ -23,7 +23,6 @@ import ibis.util as util
 import ibis.compat as compat
 import ibis
 
-
 GLOBAL_TMP_DB = os.environ.get('IBIS_TEST_TMP_DB',
                                '__ibis_tmp_{0}'.format(util.guid()))
 
@@ -118,12 +117,7 @@ class ImpalaE2E(object):
         cls.test_data_db = env.test_data_db
         cls.tmp_dir = env.tmp_dir
         cls.tmp_db = env.tmp_db
-
-        try:
-            cls.alltypes = cls.con.table('functional_alltypes')
-        except:
-            pass
-
+        cls.alltypes = cls.con.table('functional_alltypes')
         cls.db = cls.con.database(env.test_data_db)
 
         if not cls.con.exists_database(cls.tmp_db):
@@ -140,13 +134,14 @@ class ImpalaE2E(object):
             # reduce test flakiness
             try:
                 cls.con.drop_database(cls.tmp_db, force=True)
-                break
-            except:
+            except Exception:
                 i += 1
                 if i >= retries:
                     raise
 
                 time.sleep(0.1)
+            else:
+                break
 
     def setUp(self):
         self.temp_databases = []
