@@ -12,7 +12,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# flake8: noqa=E402
+import unittest
 
 from copy import copy
 import gc
@@ -23,19 +23,21 @@ import pandas as pd
 from posixpath import join as pjoin
 import pytest
 
-pytest.importorskip('sqlalchemy')
-pytest.importorskip('impala.dbapi')
-
 from ibis.expr.tests.mocks import MockConnection
-from ibis.compat import unittest, mock
-from ibis.impala import ddl
-from ibis.impala.compat import HS2Error, ImpylaError
-from ibis.impala.client import build_ast
-from ibis.impala.tests.common import ENV, ImpalaE2E, connect_test
-from ibis.tests.util import assert_equal
+from ibis.compat import mock
 import ibis.common as com
 import ibis.expr.types as ir
 import ibis.util as util
+from ibis.tests.util import assert_equal
+
+pytest.importorskip('hdfs')
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
+from ibis.impala import ddl  # noqa: E402
+from ibis.impala.compat import HS2Error, ImpylaError  # noqa: E402
+from ibis.impala.client import build_ast  # noqa: E402
+from ibis.impala.tests.common import ENV, ImpalaE2E, connect_test  # noqa: E402
 
 
 class TestDropTable(unittest.TestCase):
@@ -385,7 +387,7 @@ LOCATION '{0}'""".format(directory)
         schema = ibis.schema([('a', 'string'),
                               ('b', 'int32'),
                               ('c', 'double'),
-                              ('d', 'decimal(12,2)')])
+                              ('d', 'decimal(12, 2)')])
 
         stmt = ddl.CreateTableDelimited('new_table', path, schema,
                                         delimiter='|',
@@ -400,7 +402,7 @@ CREATE EXTERNAL TABLE IF NOT EXISTS foo.`new_table`
 (`a` string,
  `b` int,
  `c` double,
- `d` decimal(12,2))
+ `d` decimal(12, 2))
 ROW FORMAT DELIMITED
 FIELDS TERMINATED BY '|'
 ESCAPED BY '\\'
@@ -705,7 +707,7 @@ class TestDDLE2E(ImpalaE2E, unittest.TestCase):
     def test_create_empty_table(self):
         schema = ibis.schema([('a', 'string'),
                               ('b', 'timestamp'),
-                              ('c', 'decimal(12,8)'),
+                              ('c', 'decimal(12, 8)'),
                               ('d', 'double')])
 
         table_name = _random_table_name()
@@ -1068,10 +1070,10 @@ CREATE EXTERNAL TABLE {0}
  `group2` char(10))
 ROW FORMAT DELIMITED
 FIELDS TERMINATED BY ','
-LOCATION '/tmp'"""
+LOCATION '/tmp/{1}'"""
 
         full_path = '{0}.testing_{1}'.format(self.tmp_db, util.guid())
-        sql = statement.format(full_path)
+        sql = statement.format(full_path, util.guid())
 
         self.con._execute(sql, results=False)
 
@@ -1146,8 +1148,6 @@ def _assert_table_not_exists(con, table_name, database=None):
         con.table(tname)
     except ImpylaError:
         pass
-    except:
-        raise
 
 
 def _ensure_drop(con, table_name, database=None):
