@@ -29,6 +29,7 @@ from ibis.expr.datatypes import Category
 from ibis.compat import StringIO, Decimal
 from ibis.expr.tests.mocks import MockConnection
 
+pytest.importorskip('hdfs')
 pytest.importorskip('sqlalchemy')
 pytest.importorskip('impala.dbapi')
 
@@ -219,9 +220,12 @@ class TestValueExprs(unittest.TestCase, ExprSQLTest):
 
     def test_decimal_casts(self):
         cases = [
-            (L('9.9999999').cast('decimal(38,5)'),
-             "CAST('9.9999999' AS decimal(38,5))"),
-            (self.table.f.cast('decimal(12,2)'), "CAST(`f` AS decimal(12,2))")
+            (L('9.9999999').cast('decimal(38, 5)'),
+             "CAST('9.9999999' AS decimal(38, 5))"),
+            (
+                self.table.f.cast('decimal(12, 2)'),
+                "CAST(`f` AS decimal(12, 2))"
+            ),
         ]
         self._check_expr_cases(cases)
 
@@ -478,7 +482,7 @@ END"""
             (f.nullif(f == 0),
              'nullif(`l_quantity`, `l_quantity` = 0)'),
             (f.fillna(0),
-             'isnull(`l_quantity`, CAST(0 AS decimal(12,2)))'),
+             'isnull(`l_quantity`, CAST(0 AS decimal(12, 2)))'),
         ]
         self._check_expr_cases(cases)
 
@@ -488,7 +492,7 @@ END"""
 
         cases = [
             (f.fillna(0),
-             'isnull(`l_extendedprice`, CAST(0 AS decimal(12,2)))'),
+             'isnull(`l_extendedprice`, CAST(0 AS decimal(12, 2)))'),
             (f.fillna(0.0), 'isnull(`l_extendedprice`, 0.0)'),
         ]
         self._check_expr_cases(cases)
@@ -849,7 +853,11 @@ class TestStringBuiltins(unittest.TestCase, ExprSQLTest):
 
     def test_like(self):
         cases = [
-            (self.table.string_col.like('foo%'), "`string_col` LIKE 'foo%'")
+            (self.table.string_col.like('foo%'), "`string_col` LIKE 'foo%'"),
+            (
+                self.table.string_col.like(['foo%', '%bar']),
+                "`string_col` LIKE 'foo%' OR `string_col` LIKE '%bar'"
+            )
         ]
         self._check_expr_cases(cases)
 
@@ -998,8 +1006,8 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
         table = self.con.table('tpch_lineitem')
 
         expr = table.l_quantity
-        assert expr._precision == 12
-        assert expr._scale == 2
+        assert expr.meta.precision == 12
+        assert expr.meta.scale == 2
 
         # TODO: what if user impyla version does not have decimal Metadata?
 
@@ -1144,14 +1152,14 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
             return x
 
         left, right = t.schema(), table.schema()
-        for i, (n, l, r) in enumerate(zip(left.names, left.types,
-                                          right.types)):
-            l = _clean_type(l)
-            r = _clean_type(r)
+        for i, (n, left, right) in enumerate(zip(
+                left.names, left.types, right.types)):
+            left = _clean_type(left)
+            right = _clean_type(right)
 
-            if l != r:
+            if left != right:
                 pytest.fail('Value for {0} had left type {1}'
-                            ' and right type {2}'.format(n, l, r))
+                            ' and right type {2}'.format(n, left, right))
 
     def assert_cases_equality(self, cases):
         for expr, expected in cases:
@@ -1224,7 +1232,7 @@ class TestImpalaExprs(ImpalaE2E, unittest.TestCase, ExprTestCases):
 
     def test_decimal_builtins_2(self):
         d = L('5.245')
-        dc = d.cast('decimal(12,5)')
+        dc = d.cast('decimal(12, 5)')
         cases = [
             (dc % 5, Decimal('0.245')),
 
@@ -1669,3 +1677,24 @@ FROM (
 ) t0"""
 
     assert result == expected
+
+
+def test_named_from_filter_groupby():
+    t = ibis.table([('key', 'string'), ('value', 'double')], name='t0')
+    gb = t.filter(t.value == 42).groupby(t.key)
+    sum_expr = lambda t: (t.value + 1 + 2 + 3).sum()  # noqa: E731
+    expr = gb.aggregate(abc=sum_expr)
+    expected = """\
+SELECT `key`, sum(((`value` + 1) + 2) + 3) AS `abc`
+FROM t0
+WHERE `value` = 42
+GROUP BY 1"""
+    assert ibis.impala.compile(expr) == expected
+
+    expr = gb.aggregate(foo=sum_expr)
+    expected = """\
+SELECT `key`, sum(((`value` + 1) + 2) + 3) AS `foo`
+FROM t0
+WHERE `value` = 42
+GROUP BY 1"""
+    assert ibis.impala.compile(expr) == expected
