@@ -12,6 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import unittest
+
 import pytest
 
 import numpy as np
@@ -19,14 +21,14 @@ import numpy as np
 from pandas.util.testing import assert_frame_equal
 import pandas as pd
 
-from ibis.compat import unittest
+import ibis
+import ibis.expr.datatypes as dt
+import ibis.expr.types as ir
+
 from ibis.common import IbisTypeError
 from ibis.impala.pandas_interop import pandas_to_ibis_schema, DataFrameWriter
 from ibis.impala.tests.common import ImpalaE2E
-import ibis.expr.datatypes as dt
-import ibis.expr.types as ir
 import ibis.util as util
-import ibis
 
 
 class TestPandasTypeInterop(unittest.TestCase):
@@ -224,7 +226,7 @@ class TestPandasInterop(ImpalaE2E, unittest.TestCase):
         table = self.con.table(table_name, database=self.tmp_db)
 
         result = (table.execute()
-                  .sort_index(by='tinyint_col')
+                  .sort_values(by='tinyint_col')
                   .reset_index(drop=True))
         assert_frame_equal(result, exhaustive_df)
 
@@ -244,5 +246,14 @@ class TestPandasInterop(ImpalaE2E, unittest.TestCase):
 
         table = writer.delimited_table(path)
         df2 = table.execute()
-
         assert_frame_equal(df2, df)
+
+
+def test_timestamp_with_timezone():
+    df = pd.DataFrame({
+        'A': pd.date_range('20130101', periods=3, tz='US/Eastern')
+    })
+    schema = pandas_to_ibis_schema(df)
+    expected = ibis.schema([('A', "timestamp('US/Eastern')")])
+    assert schema.equals(expected)
+    assert schema.types[0].equals(dt.Timestamp('US/Eastern'))
