@@ -12,26 +12,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# flake8: noqa=E402
+import unittest
 
 from posixpath import join as pjoin
 
 import pytest
 
+pytest.importorskip('hdfs')
 pytest.importorskip('sqlalchemy')
 pytest.importorskip('impala.dbapi')
 
-import impala
+import impala  # noqa: E402
 
-from pandas.util.testing import assert_frame_equal
-import pandas as pd
+from pandas.util.testing import assert_frame_equal  # noqa: E402
+import pandas as pd  # noqa: E402
 
-from ibis.compat import unittest
-from ibis.impala.compat import ImpylaError
-from ibis.impala.tests.common import ImpalaE2E, ENV
-from ibis.tests.util import assert_equal
-import ibis
-import ibis.util as util
+from ibis.impala.compat import ImpylaError  # noqa: E402
+from ibis.impala.tests.common import ImpalaE2E, ENV  # noqa: E402
+from ibis.tests.util import assert_equal  # noqa: E402
+import ibis  # noqa: E402
+import ibis.util as util  # noqa: E402
 
 
 def _tmp_name():
@@ -138,6 +138,22 @@ class TestPartitioning(ImpalaE2E, unittest.TestCase):
 
         self._verify_partitioned_table(part_t, df, unique_keys)
 
+    def test_create_partitioned_table_from_expr(self):
+        t = self.con.table('functional_alltypes')
+        expr = t[t.id <= 10][['id', 'double_col', 'month', 'year']]
+        name = 'tmppart_{}'.format(util.guid())
+        try:
+            self.con.create_table(name, expr, partition=[t.year])
+        except Exception:
+            raise
+        else:
+            new = self.con.table(name)
+            expected = expr.execute().sort_values('id').reset_index(drop=True)
+            result = new.execute().sort_values('id').reset_index(drop=True)
+            assert_frame_equal(result, expected)
+        finally:
+            self.con.drop_table(name, force=True)
+
     @pytest.mark.xfail(raises=AssertionError, reason='NYT')
     def test_insert_overwrite_partition(self):
         assert False
@@ -271,7 +287,7 @@ class TestPartitioning(ImpalaE2E, unittest.TestCase):
 
     def _verify_partitioned_table(self, part_t, df, unique_keys):
         result = (part_t.execute()
-                  .sort_index(by='id')
+                  .sort_values(by='id')
                   .reset_index(drop=True)
                   [df.columns])
 
