@@ -11,30 +11,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# flake8: noqa=E402
-
+import unittest
 from posixpath import join as pjoin
 import pytest
-
-pytest.importorskip('sqlalchemy')
-pytest.importorskip('impala.dbapi')
+import pandas as pd
 
 import ibis
 
 import ibis.expr.types as ir
 
-from ibis.impala import ddl
-import ibis.impala as api
-
 from ibis.common import IbisTypeError
-from ibis.compat import unittest, Decimal
+from ibis.compat import Decimal
 from ibis.expr.datatypes import validate_type
 from ibis.expr.tests.mocks import MockConnection
-from ibis.impala.tests.common import ImpalaE2E
+
 import ibis.expr.rules as rules
 import ibis.common as com
 import ibis.util as util
 
+pytest.importorskip('hdfs')
+pytest.importorskip('sqlalchemy')
+pytest.importorskip('impala.dbapi')
+
+from ibis.impala import ddl  # noqa: E402
+import ibis.impala as api  # noqa: E402
+from ibis.impala.tests.common import ImpalaE2E  # noqa: E402
+
 
 class TestWrapping(unittest.TestCase):
 
@@ -90,9 +92,9 @@ class TestWrapping(unittest.TestCase):
             ibis_type = validate_type(t)
 
             expr = func(sv)
-            assert type(expr) == ibis_type.scalar_type()
+            assert type(expr) == type(ibis_type.scalar_type()(expr.op()))  # noqa: E501, E721
             expr = func(av)
-            assert type(expr) == ibis_type.array_type()
+            assert type(expr) == type(ibis_type.array_type()(expr.op()))  # noqa: E501, E721
 
     def test_uda_primitive_output_types(self):
         types = [
@@ -113,8 +115,10 @@ class TestWrapping(unittest.TestCase):
 
             expr1 = func(sv)
             expr2 = func(sv)
-            assert isinstance(expr1, ibis_type.scalar_type())
-            assert isinstance(expr2, ibis_type.scalar_type())
+            expected_type1 = type(ibis_type.scalar_type()(expr1.op()))
+            expected_type2 = type(ibis_type.scalar_type()(expr2.op()))
+            assert isinstance(expr1, expected_type1)
+            assert isinstance(expr2, expected_type2)
 
     def test_decimal(self):
         func = self._register_udf(['decimal(9,0)'], 'decimal(9,0)', 'test')
@@ -257,8 +261,7 @@ class TestUDFE2E(ImpalaE2E, unittest.TestCase):
         result = self.con.execute(expr)
         # Hacky
         if datatype is 'timestamp':
-            import pandas as pd
-            assert type(result) == pd.tslib.Timestamp
+            assert type(result) == pd.Timestamp
         else:
             lop = literal.op()
             if isinstance(lop, ir.Literal):
