@@ -61,6 +61,7 @@ _ibis_type_to_sqla = {
 
     dt.Decimal: sa.NUMERIC,
     dt.Null: sa.types.NullType,
+    dt.Binary: sa.Binary,
 }
 
 _sqla_type_mapping = {
@@ -80,7 +81,10 @@ _sqla_type_mapping = {
     sa.CHAR: dt.String,
     sa.Text: dt.String,
     sa.TEXT: dt.String,
+    sa.BINARY: dt.Binary,
+    sa.Binary: dt.Binary,
     sa.DATE: dt.Date,
+    sa.Date: dt.Date,
     sa.types.NullType: dt.Null,
 }
 
@@ -292,7 +296,9 @@ def _exists_subquery(t, expr):
                 .projection([ir.literal(1).name(ir.unnamed)]))
 
     sub_ctx = ctx.subcontext()
-    clause = to_sqlalchemy(filtered, context=sub_ctx, exists=True)
+    clause = to_sqlalchemy(
+        filtered, context=sub_ctx, exists=True, params=t.params
+    )
 
     if isinstance(op, transforms.NotExistsSubquery):
         clause = sa.not_(clause)
@@ -503,17 +509,17 @@ class AlchemyContext(comp.QueryContext):
     def __init__(self, *args, **kwargs):
         self._table_objects = {}
         self.dialect = kwargs.pop('dialect', AlchemyDialect)
-        comp.QueryContext.__init__(self, *args, **kwargs)
+        super(AlchemyContext, self).__init__(*args, **kwargs)
 
     def subcontext(self):
         return type(self)(dialect=self.dialect, parent=self)
 
-    def _to_sql(self, expr, ctx):
-        return to_sqlalchemy(expr, context=ctx)
+    def _to_sql(self, expr, ctx, params=None):
+        return to_sqlalchemy(expr, context=ctx, params=params)
 
-    def _compile_subquery(self, expr):
+    def _compile_subquery(self, expr, params=None):
         sub_ctx = self.subcontext()
-        return self._to_sql(expr, sub_ctx)
+        return self._to_sql(expr, sub_ctx, params=params)
 
     def has_table(self, expr, parent_contexts=False):
         key = self._get_table_key(expr)
@@ -535,12 +541,11 @@ class AlchemyQueryBuilder(comp.QueryBuilder):
 
     select_builder = AlchemySelectBuilder
 
-    def __init__(self, expr, context=None, dialect=None):
-        if dialect is None:
-            dialect = AlchemyDialect
-
-        self.dialect = dialect
-        comp.QueryBuilder.__init__(self, expr, context=context)
+    def __init__(self, expr, context=None, dialect=None, params=None):
+        self.dialect = dialect if dialect is not None else AlchemyDialect
+        super(AlchemyQueryBuilder, self).__init__(
+            expr, context=context, params=params
+        )
 
     def _make_context(self):
         return AlchemyContext(dialect=self.dialect)
@@ -550,11 +555,11 @@ class AlchemyQueryBuilder(comp.QueryBuilder):
         return AlchemyUnion
 
 
-def to_sqlalchemy(expr, context=None, exists=False, dialect=None):
+def to_sqlalchemy(expr, context=None, exists=False, dialect=None, params=None):
     if context is not None:
         dialect = dialect or context.dialect
 
-    ast = build_ast(expr, context=context, dialect=dialect)
+    ast = build_ast(expr, context=context, dialect=dialect, params=params)
     query = ast.queries[0]
 
     if exists:
@@ -563,8 +568,10 @@ def to_sqlalchemy(expr, context=None, exists=False, dialect=None):
     return query.compile()
 
 
-def build_ast(expr, context=None, dialect=None):
-    builder = AlchemyQueryBuilder(expr, context=context, dialect=dialect)
+def build_ast(expr, context=None, dialect=None, params=None):
+    builder = AlchemyQueryBuilder(
+        expr, context=context, dialect=dialect, params=params
+    )
     return builder.get_result()
 
 
@@ -813,12 +820,13 @@ class AlchemyClient(SQLClient):
                 'Dropping tables from a different database is not yet '
                 'implemented'
             )
-        t = self.meta.tables[table_name]
+
+        t = sa.Table(table_name, self.meta)
         t.drop(checkfirst=force)
-        if t.exists():
-            raise RuntimeError(
-                'Something went wrong during DROP of table {}'.format(t.name)
-            )
+
+        assert not t.exists(), \
+            'Something went wrong during DROP of table {!r}'.format(t.name)
+
         self.meta.remove(t)
 
         qualified_name = self._fully_qualified_name(table_name, database)
@@ -861,8 +869,8 @@ class AlchemyClient(SQLClient):
     def raw_sql(self, query, results=False):
         return super(AlchemyClient, self).raw_sql(query, results=results)
 
-    def _build_ast(self, expr):
-        return build_ast(expr, dialect=self.dialect)
+    def _build_ast(self, expr, params=None):
+        return build_ast(expr, dialect=self.dialect, params=params)
 
     def _get_sqla_table(self, name, schema=None):
         return sa.Table(name, self.meta, schema=schema, autoload=True)
@@ -897,7 +905,7 @@ class AlchemySelect(Select):
         return frag
 
     def _compile_subqueries(self):
-        if len(self.subqueries) == 0:
+        if not self.subqueries:
             return
 
         for expr in self.subqueries:
@@ -1163,16 +1171,17 @@ class AlchemyUnion(Union):
 
     def compile(self):
         context = self.context
-
-        if self.distinct:
-            sa_func = sa.union
-        else:
-            sa_func = sa.union_all
+        sa_func = sa.union if self.distinct else sa.union_all
 
         left_set = context.get_compiled_expr(self.left)
-        right_set = context.get_compiled_expr(self.right)
+        left_cte = left_set.cte()
+        left_select = left_cte.select()
 
-        return sa_func(left_set, right_set)
+        right_set = context.get_compiled_expr(self.right)
+        right_cte = right_set.cte()
+        right_select = right_cte.select()
+
+        return sa_func(left_select, right_select)
 
 
 class AlchemyProxy(object):
