@@ -19,13 +19,16 @@ from .client import PostgreSQLClient, PostgreSQLDialect
 from .compiler import rewrites  # noqa
 
 
-def compile(expr):
+def compile(expr, params=None):
     """Compile an ibis expression to the PostgreSQL target.
 
     Parameters
     ----------
     expr : ibis.expr.types.Expr
         The ibis expression to compile
+    params : dict or None
+        ``dict`` mapping :class:`ibis.expr.types.ScalarParameter` objects to
+        values
 
     Returns
     -------
@@ -34,8 +37,16 @@ def compile(expr):
     Examples
     --------
     >>> import os
+    >>> import getpass
+    >>> user = os.environ.get('IBIS_POSTGRES_USER', getpass.getuser())
+    >>> password = os.environ.get('IBIS_POSTGRES_PASS')
     >>> database = os.environ.get('IBIS_TEST_POSTGRES_DB', 'ibis_testing')
-    >>> con = connect(database=database, host='localhost')
+    >>> con = connect(
+    ...     database=database,
+    ...     host='localhost',
+    ...     user=user,
+    ...     password=password
+    ... )
     >>> t = con.table('functional_alltypes')
     >>> expr = t.double_col + 1
     >>> sqla = compile(expr)
@@ -43,7 +54,7 @@ def compile(expr):
     SELECT t0.double_col + %(param_1)s AS tmp
     FROM functional_alltypes AS t0
     """
-    return to_sqlalchemy(expr, dialect=PostgreSQLDialect)
+    return to_sqlalchemy(expr, dialect=PostgreSQLDialect, params=params)
 
 
 def connect(
@@ -78,8 +89,16 @@ def connect(
     Examples
     --------
     >>> import os
+    >>> import getpass
+    >>> user = os.environ.get('IBIS_POSTGRES_USER', getpass.getuser())
+    >>> password = os.environ.get('IBIS_POSTGRES_PASS')
     >>> database = os.environ.get('IBIS_TEST_POSTGRES_DB', 'ibis_testing')
-    >>> con = connect(database=database, host='localhost')
+    >>> con = connect(
+    ...     database=database,
+    ...     host='localhost',
+    ...     user=user,
+    ...     password=password
+    ... )
     >>> con.list_tables()  # doctest: +ELLIPSIS
     [...]
     >>> t = con.table('functional_alltypes')
