@@ -12,25 +12,21 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# flake8: noqa=E402
-
-import ibis
+import unittest
 
 import pytest
 
+import ibis
+import ibis.expr.api as api
+import ibis.expr.operations as ops
+
+from ibis.expr.tests.mocks import MockConnection
+
 pytest.importorskip('sqlalchemy')
 pytest.importorskip('impala.dbapi')
 
-from ibis.impala.compiler import build_ast, to_sql
-
-from ibis import impala
-
-from ibis.expr.tests.mocks import MockConnection
-from ibis.compat import unittest
-import ibis.common as com
-
-import ibis.expr.api as api
-import ibis.expr.operations as ops
+from ibis.impala.compiler import build_ast, to_sql  # noqa: E402
+from ibis import impala  # noqa: E402
 
 
 class TestASTBuilder(unittest.TestCase):
@@ -273,6 +269,7 @@ def _get_query(expr):
     ast = build_ast(expr)
     return ast.queries[0]
 
+
 nation = api.table([
     ('n_regionkey', 'int32'),
     ('n_nationkey', 'int32'),
@@ -809,9 +806,11 @@ class TestSelectSQL(unittest.TestCase, ExprTestCases):
         assert result == expected
 
     def test_nameless_table(self):
-        # Ensure that user gets some kind of sensible error
+        # Generate a unique table name when we haven't passed on
         nameless = api.table([('key', 'string')])
-        self.assertRaises(com.RelationError, to_sql, nameless)
+        assert to_sql(nameless) == 'SELECT *\nFROM {}'.format(
+            nameless.op().name
+        )
 
         with_name = api.table([('key', 'string')], name='baz')
         result = to_sql(with_name)
@@ -2298,7 +2297,8 @@ def test_pushdown_with_or():
     expected = """\
 SELECT *
 FROM functional_alltypes
-WHERE (`double_col` > 3.14) AND (locate('foo', `string_col`) - 1 >= 0) AND
+WHERE `double_col` > 3.14 AND
+      locate('foo', `string_col`) - 1 >= 0 AND
       (((`int_col` - 1) = 0) OR (`float_col` <= 1.34))"""
     assert result == expected
 
